@@ -44,8 +44,10 @@ class FieldKind(graphene.Enum):
 
 
 @functools.singledispatch
-def get_field_schema(field):
-    raise Exception("Don't know how to convert the Django field %s (%s)" % (field, field.__class__))
+def get_field_schema(field) -> dict:
+    raise Exception(
+        "Don't know how to convert the Django field {} ({})".format(field, field.__class__)
+    )
 
 
 @get_field_schema.register(models.CharField)
